from otree.api import BaseConstants, BaseSubsession, BaseGroup, BasePlayer, models

author = 'josephnoblepal@gmail.com'

doc = """
QVSR Model Demo
"""

categories = [
    {"code": "q_1", "label": "Job opportunities for youths"},
    {"code": "q_2", "label": "Water supply in your location"},
    {"code": "q_3", "label": "Social Amenities"},
    {"code": "q_4", "label": "Road network"},
    {"code": "q_5", "label": "Agricultural products and veterinary services"},
]


class Constants(BaseConstants):
    name_in_url = 'qvsr'
    survey_default_title = "QVSR Survey Demo"
    total_vote_credits = 100
    players_per_group = None
    num_rounds = 1
    categories = sorted(categories, key=lambda x: x['label'])


class Subsession(BaseSubsession):
    def creating_session(self):
        if 'Survey_Title' in self.session.config:
            self.session.config['survey_title'] = self.session.config['Survey_Title']
        else:
            self.session.config['survey_title'] = Constants.survey_default_title

    def vars_for_admin_players(self):
        return dict(payoffs='none')


class Group(BaseGroup):
    pass


class Player(BasePlayer):
    participant_name = models.StringField(label='What is your name?')
    participant_age = models.IntegerField(label='How old are you?')
    location = models.CharField(label='What is your location?')

    for category in Constants.categories:
        locals()[category['code']] = models.IntegerField(label=category['label'], min=-10, max=10, initial=0)

    del category

    def get_player_data(self):
        res = dict()
        p = self
        question_names = []
        answers = []

        category_labels = []

        for c in categories:
            question_names.append(c['code'])
            category_labels.append(c['label'])

        for q in question_names:
            answers.append(getattr(p, q) * getattr(p, q))  # find the quadratic value of the answer

        res['answers'] = dict(zip(category_labels, answers))

        return res



from setuptools import setup, find_packages

setup(
    name='otree-qvsr',
    version='0.1',
    description='QVSR package',
    author='Joseph Noblepal',
    author_email='josephnoblepal@gmail.com',
    license='MIT',
    packages=find_packages(),
    zip_safe=False,
    install_requires=[],  # add any additional packages that
    # needs to be installed along with your package. Eg: 'caer'
    keywords=['python', 'qvsr', 'otree'],
    classifiers=[
        "Development Status :: 1 - Planning",
        "Intended Audience :: Education",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 3",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
        "Operating System :: Unix",
    ]
)



from qvsr._builtin import Page
from qvsr.models import Constants


class SP0(Page):
    template_name = 'qvsr/SP.html'
    form_model = 'player'
    form_fields = ['participant_name', 'participant_age', 'location']


class PreferencesPage(Page):
    form_model = 'player'
    form_fields = []

    for category in Constants.categories:
        form_fields.append(category['code'])

    def generate_categories(self):
        my_categories = []

        for c in Constants.categories:
            my_categories.append({'code': c['code'], 'label': c['label']})
        return my_categories

    def vars_for_template(self):
        return {'categories': self.generate_categories(),
                'total_votes': Constants.total_vote_credits
                }

    def js_vars(self):
        return dict(
            totalVotes=Constants.total_vote_credits
        )

    def error_message(self, values):
        sum = 0

        for v in list(values.values()):
            sum += v ** 2

        if sum > Constants.total_vote_credits:
            return 'You have exceeded the maximum number of votes.'
        elif sum == 0:
            return 'You have to select at least one category.'
        else:
            pass


class SurveyPage(Page):
    form_model = 'player'

    def generate_filtered_categories(self):
        categories = []
        for c in Constants.categories:
            if getattr(self.player, c['code']) != 0:
                categories.append({'code': c['code'], 'label': c['label'], 'questions': c['questions']})
        self.participant.vars['categories'] = categories

        return categories

    def get_form_fields(self):
        form_fields = []
        for c in self.generate_filtered_categories():
            for q in c['questions']:
                form_fields.append(q['name'])
        return form_fields

    def error_message(self, values):
        categories_sums = []
        for c in self.participant.vars['categories']:
            sum = 0
            for q in c['questions']:
                sum += values[q['name']] ** 2
            categories_sums.append(sum)
            if sum > 100:
                break
        if any(x > 100 for x in categories_sums):
            return 'You have exceeded the maximum number of votes.'

    def vars_for_template(self):

        return {
            'categories': self.participant.vars['categories'],
            'total_votes': Constants.total_vote_credits
        }

    def js_vars(self):
        return dict(
            totalVotes=Constants.total_vote_credits
        )


class EndPage(Page):
    pass


page_sequence = [SP0, PreferencesPage, EndPage]
